package simulator;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class editconf implements ActionListener
{
	String title="editconf";
	String help="<html>editconf converts generic structure format to .gro, .g96 or .pdb.<br><br>The box can be modified with options -box, -d and -angles. Both -box and -d<br>will center the system in the box.<br><br>Refer to GUIMACS Help for a detailed description on editconf.</html>";
	String[] flags=new String[]{"-f","-n","-o","-mead","-bf"};
	String[] labelText=new String[]{"Input gro file","*Input ndx file","*Output gro file", "*Output pqr file", "*Input data file"};
	String[] toolTips=new String[]{"Option: -f ; Filename: conf.gro ; Type: Input ; Description: Generic structure: gro g96 pdb tpr tpb tpa xml", "Option: -n ; Filename: index.ndx ; Type: Input, Opt. ; Description: Index file","Option: -o ; Filename: out.gro ; Type: Output, Opt. ; Description: Generic structure: gro g96 pdb xml", "Option: -mead ; Filename: mead.pqr ; Type: Output, Opt. ; Description: Coordinate file for MEAD","Option: -bf ; Filename: bfact.dat ; Type: Input, Opt. ; Description: Generic data file"}; 
	String[] optFlags=new String[]{"-d","-box"};
	String[] optText=new String[]{"Distance between the solute and the box","Box vector lengths a,b,c (space separated)"};
	output out;
	JLabel[] labels,optlabels;
	JButton[] buttons;
	JPanel[] panels,optPanels,reqPanes,optPanes;
	JPanel pane;
	public JTextField[] textboxes,opttboxes;
	public JCheckBox[] checkboxes,optcboxes;
	JFileChooser jfc=new JFileChooser();
	JButton output=new JButton("View Output");
	JButton bLog,bClg,bCbk,bOut;
	JTextArea tbRun=new JTextArea(5,60);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	Font f=new Font("Serif",Font.BOLD,20);
	JCheckBox ccbox=new JCheckBox("-c");
	JLabel clabel=new JLabel("Center molecule in box (implied by -box and -d)");
	JFrame opf;
	int x=0,y=1;
	String comm=null,pwd=null;
	public editconf(JScrollPane content,Dimension frSize,String runDir)
	{
		pwd=runDir;
		jfc.setCurrentDirectory(new java.io.File(pwd));
		tbRun.setLineWrap(true);
		tbRun.setWrapStyleWord(true);
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		optlabels=new JLabel[optFlags.length];
		textboxes=new JTextField[flags.length];
		opttboxes=new JTextField[optFlags.length];
		checkboxes=new JCheckBox[flags.length];
		optcboxes=new JCheckBox[optFlags.length];
		panels=new JPanel[flags.length];
		optPanels=new JPanel[optFlags.length];
		optPanes=new JPanel[optFlags.length+1];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		pane=new JPanel(gb);
		
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
		
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters [ * denotes optional parameters ]"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JPanel optPane=new JPanel();
		optPane.setPreferredSize(new Dimension(600,150));
		optPane.setBorder(BorderFactory.createTitledBorder("Special Parameters"));
		cgc.gridx=0;
		cgc.gridy=1;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(optPane,cgc);
		centerPane.add(optPane);

		SpringLayout layout=new SpringLayout();
		optPane.setLayout(layout);
		optPanes[0]=new JPanel();
		optPanes[0].add(ccbox);
		optPanes[0].add(clabel);
		layout.putConstraint(SpringLayout.NORTH,optPanes[0],10,SpringLayout.NORTH,optPane);
		layout.putConstraint(SpringLayout.WEST,optPanes[0],10,SpringLayout.WEST,optPane);
		optPane.add(optPanes[0]);
		for(int i=0;i<optFlags.length;i++){
			opttboxes[i]=new JTextField(10);
			optcboxes[i]=new JCheckBox(optFlags[i]);
			optlabels[i]=new JLabel(optText[i]);
			optPanes[i+1]=new JPanel();
			optPanes[i+1].add(optcboxes[i]);
			optPanes[i+1].add(optlabels[i]);
			optPanes[i+1].add(opttboxes[i]);
			layout.putConstraint(SpringLayout.NORTH,optPanes[i+1],5,SpringLayout.SOUTH,optPanes[i]);
			layout.putConstraint(SpringLayout.WEST,optPanes[i+1],0,SpringLayout.WEST,optPanes[i]);
			optPane.add(optPanes[i+1]);
		}

			
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=2;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}

		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText(title+" ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		JScrollPane runsp=new JScrollPane(tbRun,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		runPane.add(runsp);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=4;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
		
		gc.gridx=0;
		gc.gridy=5;
		gb.setConstraints(output,gc);
		pane.add(output);
		output.setEnabled(false);
		output.addActionListener(this);
		output.setActionCommand("output");
		
		content.getViewport().add(pane);
		content.setAutoscrolls(true);
		
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		
		if(btn.equals("ok"))
		{
			String command=title+" ";
			for(int i=0;i<flags.length;i++)
			{
				if(checkboxes[i].isSelected())
				{
					command=command+flags[i]+" "+textboxes[i].getText()+" ";
				}
			}
			for(int i=0;i<optFlags.length;i++)
			{
				if(optcboxes[i].isSelected())
				{
					command=command+optFlags[i]+" "+opttboxes[i].getText().trim()+" ";
				}
			}
			if(ccbox.isSelected())
			{
				command=command+"-c ";
			}
			tbRun.setText(command);
		}
		if(btn.equals("run"))
		{
			comm=tbRun.getText().trim();
			if(comm.length()==0)
			{
				JOptionPane.showMessageDialog(null,"No run string found.\nPlease enter the run string manually\nor enter values in appropriate boxes above and click \"OK\"","Error",JOptionPane.ERROR_MESSAGE);
				tbRun.requestFocus();
			}
			else
			{
				Thread runner=new Thread()
				{
					public void run()
					{
						try
						{
							BufferedWriter bw=new BufferedWriter(new FileWriter(pwd+"/ecnf.exp"));
							bw.write("#!/bin/expect\n");
							bw.write("spawn -noecho "+comm+"\n");
							bw.write("set timeout -1\n");
							bw.write("expect EOF\n");
							bw.close();
							String[] runcomm={"/bin/sh","-c","expect ecnf.exp >ecnf.log"};
							Process myProc=Runtime.getRuntime().exec(runcomm,null,(new File(pwd)));
							int evl=myProc.waitFor();
							(new File(pwd+"/ecnf.exp")).delete();
							JOptionPane.showMessageDialog(null,"GUIMACS finished running editconf.\nHowever this may not mean you got the results you are looking for.\nPlease look at the log files to make sure that you got the required results.\n\n"+pwd+"/ecnf.log - output log\n"+pwd+"/ecnf.err - error log","Done",JOptionPane.INFORMATION_MESSAGE);
							output.setEnabled(true);
						}
						catch(IOException ioe)
						{
							JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
						}
						catch(InterruptedException inte)
						{
							JOptionPane.showMessageDialog(null,inte,"Interrupted Exception",JOptionPane.ERROR_MESSAGE);
						}
					}
				};
				runner.start();
			}
		}
		if(btn.equals("output"))
		{
			opf=new JFrame("Options");
			opf.setVisible(true);
			opf.setResizable(false);
			opf.setLocationRelativeTo(null);
			Box box=Box.createVerticalBox();
			opf.getContentPane().add(box);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			JPanel pView=new JPanel(new GridLayout(3,3));
			bLog=new JButton("View Output Log File");
			bClg=new JButton("Clean up Log Files");
			bCbk=new JButton("Remove backed up files");
			bOut=new JButton("Close This Window");
			bLog.addActionListener(this);
			bLog.setActionCommand("log");
			bClg.addActionListener(this);
			bClg.setActionCommand("clog");
			bCbk.addActionListener(this);
			bCbk.setActionCommand("cbk");
			bOut.addActionListener(this);
			bOut.setActionCommand("out");
			pView.add(bLog);
			if(!(new File(pwd+"/ecnf.log")).exists())
			{
				bLog.setEnabled(false);
			}
			pView.add(bClg);
			if(!(new File(pwd+"/ecnf.log")).exists())
			{
				bClg.setEnabled(false);
			}
			pView.add(bCbk);
			pView.add(bOut);
			box.add(pView);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			opf.pack();
		}
		if(btn.equals("log"))
		{
			String filename=pwd+"/ecnf.log";
			out=new output(filename);
			out.show();
		}
		if(btn.equals("clog"))
		{
			(new File(pwd+"/ecnf.log")).delete();
			JOptionPane.showMessageDialog(null,"Operation Successful");
			bClg.setEnabled(false);
			bLog.setEnabled(false);
		}
		if(btn.equals("cbk"))
		{
			try{
				String[] runcomm={"/bin/sh","-c","rm -f "+pwd+"/\\#*"};
				Process proc=Runtime.getRuntime().exec(runcomm);
				proc.waitFor();
				JOptionPane.showMessageDialog(null,"Cleaned up back up files");
				bCbk.setEnabled(false);
			}
			catch(IOException ioe)
			{
				JOptionPane.showMessageDialog(null,ioe);
			}
			catch(InterruptedException inte)
			{
				JOptionPane.showMessageDialog(null,inte);
			}
		}
		if(btn.equals("out"))
		{
			opf.hide();
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
