package simulator;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class genion implements ActionListener
{
	String title="genion";
	String help="<html>genion replaces solvent molecules by monoatomic ions at the position of the<br>first atoms with the most favorable electrostatic potential or at random. The<br>potential is calculated on all atoms, using normal GROMACS particle based<br>methods (in contrast to other methods based on solving the Poisson-Boltzmann<br>equation).<br><br>Refer to GUIMACS Help for a detailed description on genion.</html>";
	String[] flags=new String[]{"-s","-table","-n","-o","-g","-pot"};
	String[] labelText=new String[]{"Input tpr file","*Input xvg file", "*Input ndx file","Output gro file","Output log file","*Output pdb file"};
	String[] toolTips=new String[]{"Option: -s ; Filename: topol.tpr ; Type: Input ; Description: Generic run input: tpr tpb tpa xml","Option: -table ; Filename: table.xvg ; Type: Input, Opt. ; Description: xvgr/xmgr file","Option: -n ; Filename: index.ndx ; Type: Input, Opt. ; Description: Index file","Option: -o ; Filename: out.gro ; Type: Output ; Description: Generic structure: gro g96 pdb xml","Option: -g ; Filename: genion.log ; Type: Output ; Description: Log file","Option: -pot ; Filename: pot.pdb ; Type: Output, Opt. ; Description: Protein data bank file"};
	String[] optFlags=new String[]{"-np","-pq","-pname","-nn","-nq","-nname"};
	String[] optText=new String[]{"Number of positive ions","Charge of the positive ion","Name of the positive ion","Number of negative ions","Charge of the negative ion","Name of the negative ion"};
	output out;
	JLabel[] labels,optlabels;
	JButton[] buttons;
	JPanel[] panels,reqPanes,optPanes;
	public JTextField[] textboxes,opttboxes;
	public JCheckBox[] checkboxes,optcboxes;
	JFileChooser jfc=new JFileChooser();
	JButton output=new JButton("View Output");
	JButton bLog,bClg,bCbk,bOut;
	JTextArea tbRun=new JTextArea(5,60);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	InputStreamHandler ish=new InputStreamHandler();
	Font f=new Font("Serif",Font.BOLD,20);
	JFrame opf;
	int x=0,y=1;
	String comm=null,pwd=null;
	public genion(JScrollPane content, Dimension frSize, String runDir)
	{
		pwd=runDir;
		jfc.setCurrentDirectory(new java.io.File(pwd));
		tbRun.setLineWrap(true);
		tbRun.setWrapStyleWord(true);
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		optlabels=new JLabel[optFlags.length];
		textboxes=new JTextField[flags.length];
		opttboxes=new JTextField[optFlags.length];
		checkboxes=new JCheckBox[flags.length];
		optcboxes=new JCheckBox[optFlags.length];
		panels=new JPanel[flags.length];
		optPanes=new JPanel[flags.length];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		JPanel pane=new JPanel(gb);
		content.getViewport().add(pane);
		content.setAutoscrolls(true);
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
		
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=1;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}
		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText("genion ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		JScrollPane runsp=new JScrollPane(tbRun,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		runPane.add(runsp);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=3;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
		
		gc.gridx=0;
		gc.gridy=4;
		gb.setConstraints(output,gc);
		pane.add(output);
		output.setEnabled(false);
		output.addActionListener(this);
		output.setActionCommand("output");
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		
		if(btn.equals("ok"))
		{
			String command="genion ";
			for(int i=0;i<flags.length;i++)
			{
				if(checkboxes[i].isSelected())
				{
					command=command+flags[i]+" "+textboxes[i].getText()+" ";
				}
			}
			tbRun.setText(command);
			command=title+" ";
		}
		if(btn.equals("run"))
		{
			comm=tbRun.getText().trim();
			if(comm.length()==0)
			{
				JOptionPane.showMessageDialog(null,"No run string found.\nPlease enter the run string manually\nor enter values in appropriate boxes above and click \"OK\"","Error",JOptionPane.ERROR_MESSAGE);
				tbRun.requestFocus();
			}
			else
			{
				Thread runner=new Thread()
				{
					public void run()
					{
						try
						{
							BufferedWriter bw=new BufferedWriter(new FileWriter(pwd+"/gion.exp"));
							bw.write("#!/bin/expect\n");
							bw.write("spawn -noecho "+comm+"\n");
							bw.write("set timeout -1\n");
							bw.write("expect EOF\n");
							bw.close();
							String[] runcomm={"/bin/sh","-c","expect gion.exp >gion.log"};
							Process myProc=Runtime.getRuntime().exec(runcomm,null,(new File(pwd)));
							JOptionPane.showMessageDialog(null,"genion started successfully.\nYou will be intimated when the process finishes.","Started...",JOptionPane.INFORMATION_MESSAGE);
							int evl=myProc.waitFor();
							(new File(pwd+"/gion.exp")).delete();
							JOptionPane.showMessageDialog(null,"GUIMACS finished running genion.\nHowever this may not mean you got the results you are looking for.\nPlease look at the log files to make sure that you got the required results.\n\n"+pwd+"/gion.log - output log","Done",JOptionPane.INFORMATION_MESSAGE);
							output.setEnabled(true);
						}
						catch(IOException ioe)
						{
							JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
						}
						catch(InterruptedException inte)
						{
							JOptionPane.showMessageDialog(null,inte,"Interrupted Exception",JOptionPane.ERROR_MESSAGE);
						}
					}
				};
				runner.start();
			}
		}
		if(btn.equals("output"))
		{
			opf=new JFrame("Options");
			opf.setVisible(true);
			opf.setResizable(false);
			opf.setLocationRelativeTo(null);
			Box box=Box.createVerticalBox();
			opf.getContentPane().add(box);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			JPanel pView=new JPanel(new GridLayout(3,3));
			bLog=new JButton("View Output Log File");
			bClg=new JButton("Clean up Log Files");
			bCbk=new JButton("Remove backed up files");
			bOut=new JButton("Close This Window");
			bLog.addActionListener(this);
			bLog.setActionCommand("log");
			bClg.addActionListener(this);
			bClg.setActionCommand("clog");
			bCbk.addActionListener(this);
			bCbk.setActionCommand("cbk");
			bOut.addActionListener(this);
			bOut.setActionCommand("out");
			pView.add(bLog);
			if(!(new File(pwd+"/gion.log")).exists())
			{
				bLog.setEnabled(false);
			}
			pView.add(bClg);
			if(!(new File(pwd+"/gion.log")).exists())
			{
				bClg.setEnabled(false);
			}
			pView.add(bCbk);
			pView.add(bOut);
			box.add(pView);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			opf.pack();
		}
		if(btn.equals("log"))
		{
			String filename=pwd+"/gion.log";
			out=new output(filename);
			out.show();
		}
		if(btn.equals("clog"))
		{
			(new File(pwd+"/gion.log")).delete();
			JOptionPane.showMessageDialog(null,"Operation Successful");
			bClg.setEnabled(false);
			bLog.setEnabled(false);
		}
		if(btn.equals("cbk"))
		{
			try{
				String[] runcomm={"/bin/sh","-c","rm -f "+pwd+"/\\#*"};
				Process proc=Runtime.getRuntime().exec(runcomm);
				proc.waitFor();
				JOptionPane.showMessageDialog(null,"Cleaned up back up files");
				bCbk.setEnabled(false);
			}
			catch(IOException ioe)
			{
				JOptionPane.showMessageDialog(null,ioe);
			}
			catch(InterruptedException inte)
			{
				JOptionPane.showMessageDialog(null,inte);
			}
		}
		if(btn.equals("out"))
		{
			opf.hide();
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
