package simulator;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class mdrun_mpi implements ActionListener
{
	String title="mdrun_mpi";
	String help="<html>The mdrun program is the main computational chemistry engine within GROMACS.<br>Obviously, it performs Molecular Dynamics simulations, but it can also perform<br>Brownian Dynamics and Langevin Dynamics as well as Conjugate Gradient or<br>Steepest Descents energy minimization. Normal mode analysis is another option.<br>In this case mdrun builds a Hessian matrix from single conformation.<br><br>For usual Normal Modes-like calculations, make sure that the structure<br>provided is properly energy-minimised. The generated matrix can be<br>diagonalized by g_nmeig.<br><br>Refer to GUIMACS Help for a detailed description on mdrun.</html>";
	String[] flags=new String[]{"-s","-o","-x","-c","-e","-g","-dgdl","-field","-table","-tablep","-rerun","-tpi","-ei","-eo","-j","-jo","-ffout","-devout","-runav","-pi","-po","-pd","-pn","-mtx","-dn"};
	String[] labelText=new String[]{"Input tpr file","Output trr file","*Output xtc file", "Output gro file","Output edr file","Output log file","*Output xvg file","*Output xvg file","*Input xvg file","*Input xvg file","*Input xtc file","*Output xvg file","*Input edi file","*Output edo file","*Input gct file","*Output gct file","*Output xvg file","*Output xvg file","*Output xvg file","*Input ppa file","*Output ppa file","*Output pdo file","*Input ndx file","*Output mtx file","*Output ndx file"};
	String[] toolTips=new String[]{"Option: -s ; Filename: topol.tpr ; Type: Input ; Description: Generic run input: tpr tpb tpa xml","Option: -o ; Filename: traj.trr ; Type: Output ; Description: Full precision trajectory: trr trj","Option: -x ; Filename: traj.xtc ; Type: Output, Opt. ; Description: Compressed trajectory (portable xdr format)","Option: -c ; Filename: confout.gro ; Type: Output ; Description: Generic structure: gro g96 pdb xml","Option: -e ; Filename: ener.edr ; Type: Output ; Description: Generic energy: edr ene","Option: -g ; Filename: md.log ; Type: Output ; Description: Log file","Option: -dgdl ; Filename: dgdl.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -field ; Filename: field.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -table ; Filename: table.xvg ; Type: Input, Opt. ; Description: xvgr/xmgr file","Option: -table ; Filename: ptablep.xvg ; Type: Input, Opt. ; Description: xvgr/xmgr file","Option: -rerun ; Filename: rerun.xtc ; Type: Input, Opt. ; Description: Generic trajectory: xtc trr trj gro g96 pdb","Option: -tpi ; Filename: tpi.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -ei ; Filename: sam.edi ; Type: Input, Opt. ; Description: ED sampling input","Option: -eo ; Filename: sam.edo ; Type: Output, Opt. ; Description: ED sampling output","Option: -j ; Filename: wham.gct ; Type: Input, Opt. ; Description: General coupling stuff","Option: -jo ; Filename: bam.gct ; Type: Output, Opt. ; Description: General coupling stuff","Option: -ffout ; Filename: gct.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -devout ; Filename: deviatie.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -runav ; Filename: runaver.xvg ; Type: Output, Opt. ; Description: xvgr/xmgr file","Option: -pi ; Filename: pull.ppa ; Type: Input, Opt. ; Description: Pull parameters","Option: -po ; Filename: pullout.ppa ; Type: Output, Opt. ; Description: Pull parameters","Option: -pd ; Filename: pull.pdo ; Type: Output, Opt. ; Description: Pull data output","Option: -pn ; Filename: pull.ndx ; Type: Input, Opt. ; Description: Index file","Option: -mtx ; Filename: nm.mtx ; Type: Output, Opt. ; Description: Hessian matrix","Option: -dn ; Filename: dipole.ndx ; Type: Output, Opt. ; Description: Index file"};
	String[] optFlags=new String[]{"-[no]h","-[no]X","-nice","-[no]xvgr","-[no]free","-[no]rms","-cutoff"};
//	String[] optText=new String[]{"
	output out;
	JLabel[] labels;
	JButton[] buttons;
	JPanel[] panels,reqPanes,optionPanes;
	JPanel pane;
	public JTextField[] textboxes;
	public JCheckBox[] checkboxes;
	JCheckBox jcbc,jcbC,jcbN,jcbn;
	JTextField jtfc,jtfn;
	JFileChooser jfc=new JFileChooser();
	JButton output=new JButton("View Output");
	JButton bLog,bErr,bClg,bCbk,bOut,bEnd;
	JTextArea tbRun=new JTextArea(5,60);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	InputStreamHandler ish=new InputStreamHandler();
	Font f=new Font("Serif",Font.BOLD,20);
	JFrame opf;
	int x=0,y=1;
	String comm=null,pwd=null;
	public mdrun_mpi(JScrollPane content,Dimension frSize, String runDir)
	{
		pwd=runDir;
		jfc.setCurrentDirectory(new java.io.File(pwd));
		tbRun.setLineWrap(true);
		tbRun.setWrapStyleWord(true);
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		textboxes=new JTextField[flags.length];
		checkboxes=new JCheckBox[flags.length];
		panels=new JPanel[flags.length];
		optionPanes=new JPanel[flags.length];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		pane=new JPanel(gb);
		
		content.getViewport().add(pane);
		content.setAutoscrolls(true);
		
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
		
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters [ * denotes optional parameters ]"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JPanel mpiPane=new JPanel();
		mpiPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("MPI Options"),BorderFactory.createEmptyBorder(10,10,10,10)));
		cgc.gridx=0;
		cgc.gridy=1;
		cgb.setConstraints(mpiPane,cgc);
		centerPane.add(mpiPane);

		Box mpiMain=Box.createVerticalBox();
		mpiPane.add(mpiMain);
		jcbc=new JCheckBox("c - cpu list. ( e.g. c0-3, c1,3 etc. )");
		jcbC=new JCheckBox("C - Run on all CPUs available");
		jcbn=new JCheckBox("n - node list (e.g. n0-3, n1,3 etc.)");
		jcbN=new JCheckBox("N - Run on all nodes available");
		jcbc.addActionListener(this);
		jcbn.addActionListener(this);
		jcbC.addActionListener(this);
		jcbN.addActionListener(this);
		jtfn=new JTextField(10);
		jtfn.setEnabled(false);
		jtfc=new JTextField(10);
		jtfc.setEnabled(false);

		JPanel cpu=new JPanel();
		JPanel node=new JPanel();
		cpu.add(jcbC);
		cpu.add(jcbc);
		cpu.add(jtfc);
		node.add(jcbN);
		node.add(jcbn);
		node.add(jtfn);

		mpiMain.add(cpu);
		mpiMain.add(node);
		
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=2;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}
		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText("/usr/bin/mpirun ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		JScrollPane runsp=new JScrollPane(tbRun,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		runPane.add(runsp);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=3;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
		
		gc.gridx=0;
		gc.gridy=4;
		gb.setConstraints(output,gc);
		pane.add(output);
		output.setEnabled(false);
		output.addActionListener(this);
		output.setActionCommand("output");
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		if (jcbc.isSelected()) jtfc.setEnabled(true);
		else jtfc.setEnabled(false);
		if (jcbn.isSelected()) jtfn.setEnabled(true);
		else jtfn.setEnabled(false);
		if(btn.equals("ok"))
		{
			String command="/usr/bin/mpirun ";
			if (jcbc.isSelected()) command+="c"+jtfc.getText().trim()+" ";
			if (jcbC.isSelected()) command+="C ";
			if (jcbn.isSelected()) command+="n"+jtfn.getText().trim()+" ";
			if (jcbN.isSelected()) command+="N ";
			command+="/usr/bin/mdrun_mpi ";
			for(int i=0;i<flags.length;i++)
			{
				if (checkboxes[i].isSelected())	command=command+flags[i]+" "+textboxes[i].getText()+" ";				
			}
			tbRun.setText(command);
		}
		if(btn.equals("run"))
		{
			comm=tbRun.getText().trim();
			if(comm.length()==0)
			{
				JOptionPane.showMessageDialog(null,"No run string found.\nPlease enter the run string manually\nor enter values in appropriate boxes above and click \"OK\"","Error",JOptionPane.ERROR_MESSAGE);
				tbRun.requestFocus();
			}
			else
			{
				Thread runner=new Thread()
				{
					public void run()
					{
						try
						{
							BufferedWriter bw=new BufferedWriter(new FileWriter(pwd+"/mrunmpi.exp"));
							bw.write("#!/bin/expect\n");
							bw.write("spawn -noecho "+comm+"\n");
							bw.write("set timeout -1\n");
							bw.write("expect EOF\n");
							bw.close();
							String[] runcomm={"/bin/sh","-c","expect mrunmpi.exp >mrunmpi.log"};
							Process myProc=Runtime.getRuntime().exec(runcomm,null,(new File(pwd)));
							int evl=myProc.waitFor();
							(new File(pwd+"/mrun.exp")).delete();
							JOptionPane.showMessageDialog(null,"GUIMACS finished running mdrun.\nHowever this may not mean you got the results you are looking for.\nPlease look at the log files to make sure that you got the required results.\n\n"+pwd+"/mrunmpi.log - output log\n"+pwd+"/mrunmpi.err - error log","Done",JOptionPane.INFORMATION_MESSAGE);
							output.setEnabled(true);
						}
						catch(IOException ioe)
						{
							JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
						}
						catch(InterruptedException inte)
						{
							JOptionPane.showMessageDialog(null,inte,"Interrupted Exception",JOptionPane.ERROR_MESSAGE);
						}
					}
				};
				runner.start();
			}
		}
		if(btn.equals("output"))
		{
			opf=new JFrame("Options");
			opf.setVisible(true);
			opf.setResizable(false);
			opf.setLocationRelativeTo(null);
			Box box=Box.createVerticalBox();
			opf.getContentPane().add(box);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			JPanel pView=new JPanel(new GridLayout(3,3));
			bLog=new JButton("View Output Log File");
			bClg=new JButton("Clean up Log Files");
			bCbk=new JButton("Remove backed up files");
			bOut=new JButton("Close This Window");
			bLog.addActionListener(this);
			bLog.setActionCommand("log");
			bClg.addActionListener(this);
			bClg.setActionCommand("clog");
			bCbk.addActionListener(this);
			bCbk.setActionCommand("cbk");
			bOut.addActionListener(this);
			bOut.setActionCommand("out");
			pView.add(bLog);
			if(!(new File(pwd+"/mrunmpi.log")).exists())
			{
				bLog.setEnabled(false);
			}
			pView.add(bClg);
			if(!(new File(pwd+"/mrunmpi.log")).exists() && !(new File(pwd+"/mrunmpi.err")).exists())
			{
				bClg.setEnabled(false);
			}
			pView.add(bCbk);
			pView.add(bOut);
			box.add(pView);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			opf.pack();
		}
		if(btn.equals("log"))
		{
			String filename=pwd+"/mrunmpi.log";
			out=new output(filename);
			out.show();
		}
		if(btn.equals("clog"))
		{
			(new File(pwd+"/mrunmpi.log")).delete();
			JOptionPane.showMessageDialog(null,"Operation Successful");
			bClg.setEnabled(false);
			bLog.setEnabled(false);			
		}
		if(btn.equals("cbk"))
		{
			try{
				String[] runcomm={"/bin/sh","-c","rm -f "+pwd+"/\\#*"};
				Process proc=Runtime.getRuntime().exec(runcomm);
				proc.waitFor();
				JOptionPane.showMessageDialog(null,"Cleaned up back up files");
				bCbk.setEnabled(false);
			}
			catch(IOException ioe)
			{
				JOptionPane.showMessageDialog(null,ioe);
			}
			catch(InterruptedException inte)
			{
				JOptionPane.showMessageDialog(null,inte);
			}
		}
		if(btn.equals("out"))
		{
			opf.hide();
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
