package simulator;
import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;

public class pdb2gmx extends JFrame implements ActionListener
{
	String title="pdb2gmx";
	String help="<html>This program reads a pdb file, reads some database files, adds hydrogens to<br>the molecules and generates coordinates in Gromacs (Gromos) format and a<br>topology in Gromacs format. These files can subsequently be processed to<br>generate a run input file.<br><br>Refer to GUIMACS Help for a detailed description on pdb2gmx.</html>";
	String[] flags=new String[]{"-f","-o","-p","-i","-n","-q"};
	String[] labelText=new String[]{"Input pdb file","Output gro file","Output top file", "Output itp file","*Output ndx file","*Output pdb file"};
	String[] toolTips=new String[]{"Option: -f ; Filename: eiwit.pdb ; Type: Input ; Description: Generic structure: gro g96 pdb tpr tpb tpa xml","Option: -o ; Filename: conf.gro ; Type: Output ; Description: Generic structure: gro g96 pdb xml","Option: -p ; Filename: topol.top ; Type: Output ; Description: Topology file","Option: -i ; Filename: posre.itp ; Type: Output ; Description: Include file for topology","Option: -n ; Filename: clean.ndx ; Type: Output, Opt. ; Description: Index file","Option: -q ; Filename: clean.pdb ; Type: Output, Opt. ; Description: Generic structure: gro g96 pdb xml"};
	String[] optFlags=new String[]{"-[no]h","-[no]X","-nice","-[no]xvgr","-[no]free","-[no]rms","-cutoff"};
//	String[] optText=new String[]{"
	Vector forcefields=new Vector();
	output out;
	JLabel[] labels;
	JButton[] buttons;
	JPanel[] panels,reqPanes,optionPanes;
	JPanel pane;
	public JTextField[] textboxes;
	public JCheckBox[] checkboxes;
	JFileChooser jfc=new JFileChooser();
	JButton output=new JButton("View Output");
	JButton bLog,bClg,bCbk,bOut;
	JTextArea tbRun=new JTextArea(5,60);
	JLabel helpLabel=new JLabel(help);
	JLabel titleLabel=new JLabel(title);
	Font f=new Font("Serif",Font.BOLD,20);
	JComboBox forcefield;
	JFrame opf;
	int i=0,x=0,y=1;
	String comm=null,pwd=null,nextLine=null;
	public pdb2gmx(JScrollPane content,Dimension frSize,String runDir)
	{
		pwd=runDir;
		jfc.setCurrentDirectory(new java.io.File(pwd));
		tbRun.setLineWrap(true);
		tbRun.setWrapStyleWord(true);
		buttons=new JButton[flags.length];
		labels=new JLabel[flags.length];
		textboxes=new JTextField[flags.length];
		checkboxes=new JCheckBox[flags.length];
		panels=new JPanel[flags.length];
		optionPanes=new JPanel[flags.length];
		reqPanes=new JPanel[flags.length];
		
		GridBagLayout gb=new GridBagLayout();
		GridBagConstraints gc=new GridBagConstraints();
		pane=new JPanel(gb);
		
		titleLabel.setFont(f);
		gc.gridx=0;
		gc.gridy=0;
		gc.weightx=0.2;
		gc.weighty=0.3;
		gb.setConstraints(titleLabel,gc);
		pane.add(titleLabel);
		
		gc.gridx=0;
		gc.gridy=1;
		gb.setConstraints(helpLabel,gc);
		pane.add(helpLabel);
		
		try{
			BufferedReader br=new BufferedReader(new FileReader(System.getProperty("GMXLIB")+"/FF.dat"));
			br.readLine();
			while((nextLine=br.readLine())!=null)
			{
				if(nextLine.trim().length()!=0)
				{
					forcefields.addElement(nextLine.trim());
				}
			}
			br.close();
		}
		catch(IOException ioe)
		{
			JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
		}
		forcefield=new JComboBox(forcefields);
		
		JPanel ffield=new JPanel(new FlowLayout());
		ffield.add(new JLabel("Please choose forcefield  "));
		ffield.add(forcefield);
		ffield.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Force field selection"),BorderFactory.createEmptyBorder(10,10,10,10)));
		gc.gridx=0;
		gc.gridy=2;
		gb.setConstraints(ffield,gc);
		pane.add(ffield);
		
		GridBagLayout cgb=new GridBagLayout();
		GridBagConstraints cgc=new GridBagConstraints();
		JPanel centerPane=new JPanel(cgb);
		gc.gridx=0;
		gc.gridy=3;
		gb.setConstraints(centerPane,gc);
		pane.add(centerPane);
	
		GridBagLayout rgb=new GridBagLayout();
		GridBagConstraints rgc=new GridBagConstraints();
		JPanel reqPane=new JPanel(rgb);
		JScrollPane jsp=new JScrollPane(reqPane);
		jsp.setAutoscrolls(true);
		cgc.gridx=0;
		cgc.gridy=0;
		cgb.setConstraints(jsp,cgc);
		centerPane.add(jsp);
		centerPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Required Parameters [ * denotes optional parameters ]"),BorderFactory.createEmptyBorder(10,10,10,10)));
		
		JButton bOk=new JButton("OK");
		bOk.addActionListener(this);
		bOk.setActionCommand("ok");
		cgc.gridx=0;
		cgc.gridy=1;
		cgc.gridwidth=2;
		cgc.gridheight=2;
		cgc.anchor=GridBagConstraints.LINE_END;
		cgc.insets=new Insets(5,5,0,0);
		cgb.setConstraints(bOk,cgc);
		centerPane.add(bOk);
		
		for(int i=0;i<flags.length;i++)
		{
			if(i%2==0)
			{
				y++;
				x=0;
			}
			else
			{
				x=2;
			}				
			labels[i]=new JLabel(labelText[i]);
			panels[i]=new JPanel();
			reqPanes[i]=new JPanel();
			buttons[i]=new JButton("Browse");
			buttons[i].addActionListener(this);
			buttons[i].setActionCommand(flags[i]);
			checkboxes[i]=new JCheckBox(flags[i]);
			checkboxes[i].setToolTipText(toolTips[i]);
			textboxes[i]=new JTextField(10);
			panels[i].add(checkboxes[i]);
			panels[i].add(labels[i]);
			reqPanes[i].add(textboxes[i]);
			reqPanes[i].add(buttons[i]);
			rgc.gridx=x;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(panels[i],rgc);
			reqPane.add(panels[i]);
			rgc.gridx=x+1;
			rgc.gridy=y;
			rgc.anchor=GridBagConstraints.LINE_START;
			rgb.setConstraints(reqPanes[i],rgc);
			reqPane.add(reqPanes[i]);
		}
		JPanel runPane=new JPanel(new FlowLayout());
		JButton bRun=new JButton("Run");
		tbRun.setText("pdb2gmx -ff G43a1 ");
		bRun.addActionListener(this);
		bRun.setActionCommand("run");
		JScrollPane runsp=new JScrollPane(tbRun,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		runPane.add(runsp);
		runPane.add(bRun);
		gc.gridx=0;
		gc.gridy=4;
		gb.setConstraints(runPane,gc);
		pane.add(runPane);
		
		gc.gridx=0;
		gc.gridy=5;
		gb.setConstraints(output,gc);
		pane.add(output);
		output.setEnabled(false);
		output.addActionListener(this);
		output.setActionCommand("output");
		
		content.getViewport().add(pane);
		content.setAutoscrolls(true);
		
	}
	public void actionPerformed(ActionEvent ae)
	{
		String btn=ae.getActionCommand();
		
		if(btn.equals("ok"))
		{
			String command="pdb2gmx ";
			String ff=forcefield.getSelectedItem().toString();
			String[] split=ff.split(" ");
			String fflag=split[0].substring(2,split[0].length());
			command+=" -ff "+fflag+" ";
			for(int i=0;i<flags.length;i++)
			{
				if(checkboxes[i].isSelected())
				{
					command=command+flags[i]+" "+textboxes[i].getText()+" ";
				}
			}
			tbRun.setText(command);
		}
		if(btn.equals("run"))
		{
			comm=tbRun.getText().trim();
			if(comm.length()==0)
			{
				JOptionPane.showMessageDialog(null,"No run string found.\nPlease enter the run string manually\nor enter values in appropriate boxes above and click \"OK\"","Error",JOptionPane.ERROR_MESSAGE);
				tbRun.requestFocus();
			}
			else
			{
				Thread runner=new Thread()
				{
					public void run()
					{
						try
						{
							BufferedWriter bw=new BufferedWriter(new FileWriter(pwd+"/pgmx.exp"));
							bw.write("#!/bin/expect\n");
							bw.write("spawn -noecho "+comm+"\n");
							bw.write("set timeout -1\n");
							bw.write("expect EOF\n");
							bw.close();
							String[] runcomm={"/bin/sh","-c","expect pgmx.exp >pgmx.log"};
							Process myProc=Runtime.getRuntime().exec(runcomm,null,(new File(pwd)));
							int evl=myProc.waitFor();
							(new File(pwd+"/pgmx.exp")).delete();
							BufferedReader br=new BufferedReader(new FileReader(pwd+"/pgmx.log"));
							while((nextLine=br.readLine())!=null){
								if(nextLine.indexOf("error:")!=-1){
									JOptionPane.showMessageDialog(null,"GUIMACS failed to run pdb2gmx.\nThis could mean that there is some problem with the input PDB file.\nPlease make sure the PDB file completely depicts the protein under consideration.\n\nHint:WHAT IF web interface could solve this problem.","Fatal Error",JOptionPane.ERROR_MESSAGE);
									return;
								}
								if(nextLine.startsWith("Total charge")){
									String[] fields=nextLine.split("\\.");
									String[] chrg=fields[0].split(" ");
									String sign=null;
									if(Integer.parseInt(chrg[chrg.length-1].trim())>0){
										sign="negative";
									}
									else{
										sign="positive";
									}
									JOptionPane.showMessageDialog(null,"GUIMACS finished running pdb2gmx.\nThe system requires "+chrg[chrg.length-1].trim()+" "+sign+" charges to be neutralized.\nIt is a wise idea to neutralize the system before running the simulation.\nUse genion for neutralization.\nPlease look at the log files for more details.\n\n"+pwd+"/pgmx.log - output log\n"+pwd+"/pgmx.err - error log","Done",JOptionPane.INFORMATION_MESSAGE);
									break;
								}
							}
							output.setEnabled(true);
						}
						catch(IOException ioe)
						{
							JOptionPane.showMessageDialog(null,ioe,"IOException",JOptionPane.ERROR_MESSAGE);
						}
						catch(InterruptedException inte)
						{
							JOptionPane.showMessageDialog(null,inte,"Interrupted Exception",JOptionPane.ERROR_MESSAGE);
						}
					}
				};
				runner.start();
			}
		}
		if(btn.equals("output"))
		{
			opf=new JFrame("Options");
			opf.setVisible(true);
			opf.setResizable(false);
			opf.setLocationRelativeTo(null);
			Box box=Box.createVerticalBox();
			opf.getContentPane().add(box);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			JPanel pView=new JPanel(new GridLayout(3,3));
			bLog=new JButton("View Output Log File");
			bClg=new JButton("Clean up Log Files");
			bCbk=new JButton("Remove backed up files");
			bOut=new JButton("Close This Window");
			bLog.addActionListener(this);
			bLog.setActionCommand("log");
			bClg.addActionListener(this);
			bClg.setActionCommand("clog");
			bCbk.addActionListener(this);
			bCbk.setActionCommand("cbk");
			bOut.addActionListener(this);
			bOut.setActionCommand("out");
			pView.add(bLog);
			if(!(new File(pwd+"/pgmx.log")).exists())
			{
				bLog.setEnabled(false);
			}
			pView.add(bClg);
			if(!(new File(pwd+"/pgmx.log")).exists())
			{
				bClg.setEnabled(false);
			}
			pView.add(bCbk);
			pView.add(bOut);
			box.add(pView);
			box.add(Box.createRigidArea(new Dimension(0,20)));
			opf.pack();
		}
		if(btn.equals("log"))
		{
			String filename=pwd+"/pgmx.log";
			out=new output(filename);
			out.show();
		}
		if(btn.equals("clog"))
		{
			(new File(pwd+"/pgmx.log")).delete();
			JOptionPane.showMessageDialog(null,"Operation Successful");
			bClg.setEnabled(false);
			bLog.setEnabled(false);
		}
		if(btn.equals("cbk"))
		{
			try{
				String[] runcomm={"/bin/sh","-c","rm -f "+pwd+"/\\#*"};
				Process proc=Runtime.getRuntime().exec(runcomm);
				proc.waitFor();
				JOptionPane.showMessageDialog(null,"Cleaned up back up files");
				bCbk.setEnabled(false);
			}
			catch(IOException ioe)
			{
				JOptionPane.showMessageDialog(null,ioe);
			}
			catch(InterruptedException inte)
			{
				JOptionPane.showMessageDialog(null,inte);
			}
		}
		if(btn.equals("out"))
		{
			opf.hide();
		}
		for(int i=0;i<flags.length;i++)
		{
			if(btn.equals(flags[i]))
			{
				int retval=jfc.showOpenDialog(null);
				String fname="";
				if(retval==JFileChooser.APPROVE_OPTION)
				{
					File myFile=jfc.getSelectedFile();
					fname=myFile.toString();
					textboxes[i].setText(fname);
				}
			}
		}
	
	}
}
